import { PackageJson } from 'type-fest';
import { Container } from '@supercharge/container';
import { EnvStore, ConfigStore, BootstrapperCtor, ServiceProvider, Application as ApplicationContract, Logger, ErrorHandlerCtor } from '@supercharge/contracts';
export declare class Application extends Container implements ApplicationContract {
    /**
     * Stores the application’s meta data, like the application’s root directory,
     * name of the environment file or whether the app is running in the console.
     */
    private meta;
    /**
     * Create a new application instance.
     *
     * @param basePath - the application root path
     */
    constructor(basePath: string);
    /**
     * Create a new application instance.
     *
     * @param {String} basePath - absolute path to the application’s root directory
     *
     * @returns {Application}
     */
    static createWithAppRoot(basePath: string): Application;
    /**
     * Assign the given error handler to this application instance. The error
     * handler is used to report errors on the default logging channel and
     * also to create responses for requests throwing errors.
     *
     * @param ErrorHandler
     *
     * @returns {Application}
     */
    withErrorHandler(ErrorHandler: ErrorHandlerCtor): Application;
    /**
     * Register the base bindings into the container.
     */
    private registerBaseBindings;
    /**
     * Register the base service provider into the container.
     */
    private registerBaseServiceProviders;
    /**
     * Returns the application logger instance.
     *
     * @returns {Logger}
     */
    logger(): Logger;
    /**
     * Returns the application key.
     *
     * @returns {String}
     *
     * @throws
     */
    key(): string;
    /**
     * Returns the application version defined in the `package.json` file.
     *
     * @returns {String}
     */
    version(): string | undefined;
    /**
     * Read the contents of the application’s `package.json` file.
     *
     * @returns {String}
     */
    readPackageJson(): PackageJson;
    /**
     * Resolves an absolute path to the given the given `destination` in
     * the application directory, starting at the application root.
     *
     * @param {String} destination
     *
     * @returns {String}
     */
    resolveFromBasePath(...destination: string[]): string;
    /**
     * Determine whether the application is in debug mode.
     *
     * @returns {Boolean}
     */
    debug(): boolean;
    /**
     * Resolves the absolute path from the given `destination` in the
     * application directory, starting from the application root.
     * The destination supports a glob format, like 'providers/**'.
     *
     * @param destination
     *
     * @returns {String}
     */
    resolveGlobFromBasePath(...destination: string[]): string;
    /**
     * Returns the root path of the application directory.
     *
     * @returns {String}
     */
    basePath(): string;
    /**
     * Returns the config store instance.
     *
     * @returns {ConfigStore}
     */
    config(): ConfigStore;
    /**
     * Returns an absolute path into the application’s config directory.
     *
     * @param {String} path
     *
     * @returns {String}
     */
    configPath(path?: string): string;
    /**
     * Returns an absolute path into the application’s public directory.
     *
     * @param {String} path
     *
     * @returns {String}
     */
    publicPath(path?: string): string;
    /**
     * Returns an absolute path into the application’s resources directory.
     *
     * @param {String} path
     *
     * @returns {String}
     */
    resourcePath(path?: string): string;
    /**
     * Returns an absolute path into the application’s storage directory.
     *
     * @param {String} path
     *
     * @returns {String}
     */
    storagePath(path?: string): string;
    /**
     * Returns an absolute path into the application’s database directory.
     *
     * @param {String} path
     *
     * @returns {String}
     */
    databasePath(path?: string): string;
    /**
     * Returns the env store instance.
     *
     * @returns {EnvStore}
     */
    env(): EnvStore;
    /**
     * Returns the environment file of the application. By default, this is `.env`.
     *
     * @returns {String}
     */
    environmentFile(): string;
    /**
     * Set the environment file to be loaded while bootstrapping the application.
     *
     * @param {String} file
     *
     * @returns {Application}
     */
    loadEnvironmentFrom(file: string): this;
    /**
     * Returns the path to directory of the environment file.
     * By default, this is the application's base path.
     *
     * @returns {String}
     */
    environmentPath(): string;
    /**
     * Returns the registered service providers.
     */
    serviceProviders(): ServiceProvider[];
    /**
     * Returns the registered booting callbacks.
     */
    bootingCallbacks(): Callback[];
    /**
     * Register a booting callback that runs at the beginning of the app boot.
     *
     * @param {Function} callback
     */
    booting(callback: Callback): this;
    /**
     * Register the configured user-land providers.
     */
    registerConfiguredProviders(): Promise<void>;
    /**
     * Resolve all registered user-land service providers from disk
     * and store them locally to registering and booting them.
     */
    loadConfiguredProviders(): Promise<void>;
    /**
     * Register the configured service providers.
     */
    registerServiceProviders(): void;
    /**
     * Call the `register` method on the given service `provider`.
     */
    register(provider: ServiceProvider): ServiceProvider;
    /**
     * Returns the content of the required `path`.
     *
     * @param path
     *
     * @returns {*}
     */
    require(path: string): any;
    /**
     * Boot the application’s service providers.
     */
    boot(): Promise<void>;
    /**
     * Boot the given service `provider`.
     *
     * @param provider
     */
    private bootProvider;
    /**
     * Prepare booting application by running the `bootstrappers`.
     *
     * @param {Array} bootstrappers
     */
    bootstrapWith(bootstrappers: BootstrapperCtor[]): Promise<void>;
    /**
     * Call the given booting `callbacks` for this application.
     *
     * @param {Callback[]} callbacks
     */
    runAppCallbacks(callbacks: Callback[]): Promise<void>;
    /**
     * Determine whether the application is running in the console.
     *
     * @returns {Boolean}
     */
    isRunningInConsole(): boolean;
    /**
     * Mark the application as running in the console.
     *
     * @returns {Application}
     */
    markAsRunningInConsole(): this;
}
declare type Callback = (app: Application) => unknown | Promise<unknown>;
export {};
