'use strict';
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Application = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const globby_1 = __importDefault(require("globby"));
const env_1 = require("@supercharge/env");
const config_1 = require("@supercharge/config");
const collections_1 = __importDefault(require("@supercharge/collections"));
const container_1 = require("@supercharge/container");
const goodies_1 = require("@supercharge/goodies");
const logging_1 = require("@supercharge/logging");
const routing_1 = require("@supercharge/routing");
class Application extends container_1.Container {
    /**
     * Create a new application instance.
     *
     * @param basePath - the application root path
     */
    constructor(basePath) {
        super();
        this.meta = {
            appRoot: basePath,
            serviceProviders: [],
            bootingCallbacks: [],
            isRunningInConsole: false,
            env: new env_1.Env(),
            environmentFile: '.env',
            config: new config_1.Config()
        };
        this.registerBaseBindings();
        this.registerBaseServiceProviders();
    }
    /**
     * Create a new application instance.
     *
     * @param {String} basePath - absolute path to the application’s root directory
     *
     * @returns {Application}
     */
    static createWithAppRoot(basePath) {
        return new Application(basePath);
    }
    /**
     * Assign the given error handler to this application instance. The error
     * handler is used to report errors on the default logging channel and
     * also to create responses for requests throwing errors.
     *
     * @param ErrorHandler
     *
     * @returns {Application}
     */
    withErrorHandler(ErrorHandler) {
        return (0, goodies_1.tap)(this, () => {
            this.singleton('error.handler', () => {
                return new ErrorHandler(this);
            });
        });
    }
    /**
     * Register the base bindings into the container.
     */
    registerBaseBindings() {
        this.singleton('app', () => this);
        this.singleton('container', () => this);
        this.singleton('env', () => this.env());
        this.singleton('config', () => this.config());
    }
    /**
     * Register the base service provider into the container.
     */
    registerBaseServiceProviders() {
        this.register(new routing_1.RoutingServiceProvider(this));
        this.register(new logging_1.LoggingServiceProvider(this));
    }
    /**
     * Returns the application logger instance.
     *
     * @returns {Logger}
     */
    logger() {
        return this.make('logger');
    }
    /**
     * Returns the application key.
     *
     * @returns {String}
     *
     * @throws
     */
    key() {
        if (this.config().has('app.key')) {
            return this.config().get('app.key');
        }
        throw new Error('Missing app key. Please set the APP_KEY environment variable.');
    }
    /**
     * Returns the application version defined in the `package.json` file.
     *
     * @returns {String}
     */
    version() {
        return (0, goodies_1.upon)(this.readPackageJson(), pkg => {
            return pkg.version;
        });
    }
    /**
     * Read the contents of the application’s `package.json` file.
     *
     * @returns {String}
     */
    readPackageJson() {
        return JSON.parse(fs_1.default.readFileSync(this.resolveFromBasePath('package.json')).toString());
    }
    /**
     * Resolves an absolute path to the given the given `destination` in
     * the application directory, starting at the application root.
     *
     * @param {String} destination
     *
     * @returns {String}
     */
    resolveFromBasePath(...destination) {
        return path_1.default.resolve(this.basePath(), ...destination);
    }
    /**
     * Determine whether the application is in debug mode.
     *
     * @returns {Boolean}
     */
    debug() {
        return !!this.config().get('app.debug');
    }
    /**
     * Resolves the absolute path from the given `destination` in the
     * application directory, starting from the application root.
     * The destination supports a glob format, like 'providers/**'.
     *
     * @param destination
     *
     * @returns {String}
     */
    resolveGlobFromBasePath(...destination) {
        return globby_1.default.sync(this.resolveFromBasePath(...destination))[0];
    }
    /**
     * Returns the root path of the application directory.
     *
     * @returns {String}
     */
    basePath() {
        return this.meta.appRoot;
    }
    /**
     * Returns the config store instance.
     *
     * @returns {ConfigStore}
     */
    config() {
        return this.meta.config;
    }
    /**
     * Returns an absolute path into the application’s config directory.
     *
     * @param {String} path
     *
     * @returns {String}
     */
    configPath(path) {
        return this.resolveFromBasePath('config', path !== null && path !== void 0 ? path : '');
    }
    /**
     * Returns an absolute path into the application’s public directory.
     *
     * @param {String} path
     *
     * @returns {String}
     */
    publicPath(path) {
        return this.resolveFromBasePath('public', path !== null && path !== void 0 ? path : '');
    }
    /**
     * Returns an absolute path into the application’s resources directory.
     *
     * @param {String} path
     *
     * @returns {String}
     */
    resourcePath(path) {
        return this.resolveFromBasePath('resources', path !== null && path !== void 0 ? path : '');
    }
    /**
     * Returns an absolute path into the application’s storage directory.
     *
     * @param {String} path
     *
     * @returns {String}
     */
    storagePath(path) {
        return this.resolveFromBasePath('storage', path !== null && path !== void 0 ? path : '');
    }
    /**
     * Returns an absolute path into the application’s database directory.
     *
     * @param {String} path
     *
     * @returns {String}
     */
    databasePath(path) {
        return this.resolveFromBasePath('database', path !== null && path !== void 0 ? path : '');
    }
    /**
     * Returns the env store instance.
     *
     * @returns {EnvStore}
     */
    env() {
        return this.meta.env;
    }
    /**
     * Returns the environment file of the application. By default, this is `.env`.
     *
     * @returns {String}
     */
    environmentFile() {
        return this.meta.environmentFile || '.env';
    }
    /**
     * Set the environment file to be loaded while bootstrapping the application.
     *
     * @param {String} file
     *
     * @returns {Application}
     */
    loadEnvironmentFrom(file) {
        return (0, goodies_1.tap)(this, () => {
            this.meta.environmentFile = file;
        });
    }
    /**
     * Returns the path to directory of the environment file.
     * By default, this is the application's base path.
     *
     * @returns {String}
     */
    environmentPath() {
        return this.basePath();
    }
    /**
     * Returns the registered service providers.
     */
    serviceProviders() {
        return this.meta.serviceProviders;
    }
    /**
     * Returns the registered booting callbacks.
     */
    bootingCallbacks() {
        return this.meta.bootingCallbacks;
    }
    /**
     * Register a booting callback that runs at the beginning of the app boot.
     *
     * @param {Function} callback
     */
    booting(callback) {
        return (0, goodies_1.tap)(this, () => {
            this.meta.bootingCallbacks.push(callback);
        });
    }
    /**
     * Register the configured user-land providers.
     */
    async registerConfiguredProviders() {
        await this.loadConfiguredProviders();
        this.registerServiceProviders();
    }
    /**
     * Resolve all registered user-land service providers from disk
     * and store them locally to registering and booting them.
     */
    async loadConfiguredProviders() {
        const { providers } = await this.require(this.resolveGlobFromBasePath('bootstrap/providers.**'));
        (0, collections_1.default)([].concat(providers)).forEach(Provider => {
            this.serviceProviders().push(new Provider(this));
        });
    }
    /**
     * Register the configured service providers.
     */
    registerServiceProviders() {
        this.serviceProviders().forEach(provider => {
            this.register(provider);
        });
    }
    /**
     * Call the `register` method on the given service `provider`.
     */
    register(provider) {
        return (0, goodies_1.tap)(provider, () => {
            provider.register(this);
        });
    }
    /**
     * Returns the content of the required `path`.
     *
     * @param path
     *
     * @returns {*}
     */
    require(path) {
        return (0, goodies_1.esmRequire)(path);
    }
    /**
     * Boot the application’s service providers.
     */
    async boot() {
        await (0, collections_1.default)(this.serviceProviders()).forEach(async (provider) => {
            await this.bootProvider(provider);
        });
    }
    /**
     * Boot the given service `provider`.
     *
     * @param provider
     */
    async bootProvider(provider) {
        provider.callBootingCallbacks();
        if (typeof provider.boot === 'function') {
            await provider.boot(this);
        }
        provider.callBootedCallbacks();
    }
    /**
     * Prepare booting application by running the `bootstrappers`.
     *
     * @param {Array} bootstrappers
     */
    async bootstrapWith(bootstrappers) {
        await this.runAppCallbacks(this.bootingCallbacks());
        await (0, collections_1.default)(bootstrappers).forEach(async (Bootstrapper) => {
            // TODO: resolve the instance through the container?
            await new Bootstrapper(this).bootstrap(this);
        });
    }
    /**
     * Call the given booting `callbacks` for this application.
     *
     * @param {Callback[]} callbacks
     */
    async runAppCallbacks(callbacks) {
        await (0, collections_1.default)(callbacks).forEach(async (callback) => {
            await callback(this);
        });
    }
    /**
     * Determine whether the application is running in the console.
     *
     * @returns {Boolean}
     */
    isRunningInConsole() {
        return this.meta.isRunningInConsole;
    }
    /**
     * Mark the application as running in the console.
     *
     * @returns {Application}
     */
    markAsRunningInConsole() {
        return (0, goodies_1.tap)(this, () => {
            this.meta.isRunningInConsole = true;
        });
    }
}
exports.Application = Application;
