'use strict';
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LoadConfiguration = void 0;
const fs_1 = __importDefault(require("@supercharge/fs"));
const path_1 = require("path");
const collections_1 = __importDefault(require("@supercharge/collections"));
const goodies_1 = require("@supercharge/goodies");
class LoadConfiguration {
    /**
     * Bootstrap the given application.
     *
     * @param {Application} app
     */
    async bootstrap(app) {
        [].concat(await this.loadConfigurations(app)).forEach(({ name, config }) => {
            app.config().set(name, config);
        });
    }
    /**
     * Load the configuration files from the disk.
     *
     * @param {Application} app
     *
     * @returns {object}
     */
    async loadConfigurations(app) {
        return await this.resolve(await this.loadConfigurationFilesFromDisk(app));
    }
    /**
     * Load the application’s configuration files from the local hard disk.
     *
     * @param {Application} app
     *
     * @returns {String[]}
     */
    async loadConfigurationFilesFromDisk(app) {
        return await fs_1.default.allFiles(app.configPath());
    }
    /**
     * Load the configuration files from the disk.
     *
     * @param {Application} app
     *
     * @returns {object}
     */
    async resolve(configurationFiles) {
        return (0, collections_1.default)(configurationFiles)
            .filter(configFile => {
            /**
             * This `filter` method allows us to change the property name for
             * the loaded files. Returning the new name from this method
             * will be used as the key in the resulting object.
             *
             * E.g. an `app.ts` or `app.js` file will be translated into `app`
             * as the key in the resulting object: `{ app: {…} }`.
             */
            return this.isJavascriptOrTypescript(configFile);
        })
            .map(async (configFile) => {
            /**
             * This resolves any ESM default exports to avoid a `default` property
             * in the result. Example:
             *   `{ app: { default: {…} }` -> `{ app: { {…} }`
             */
            return {
                name: (0, path_1.parse)(configFile).name,
                config: (0, goodies_1.esmRequire)(configFile)
            };
        });
    }
    /**
     * Determine whether the given file name is a JS or TS file.
     *
     * @param file
     *
     * @returns {Boolean}
     */
    isJavascriptOrTypescript(file) {
        return file.endsWith('.d.ts')
            ? false
            : ['.js', '.ts'].includes((0, path_1.extname)(file));
    }
}
exports.LoadConfiguration = LoadConfiguration;
