'use strict';
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConsoleKernel = void 0;
const fs_1 = __importDefault(require("@supercharge/fs"));
const collections_1 = __importDefault(require("@supercharge/collections"));
const goodies_1 = require("@supercharge/goodies");
const console_1 = require("@supercharge/console");
const bootstrappers_1 = require("../bootstrappers");
class ConsoleKernel {
    /**
     * Create a new console kernel instance.
     *
     * @param {Application} app
     */
    constructor(app) {
        this.meta = { app };
    }
    /**
     * Returns the application instance.
     *
     * @returns {Application}
     */
    app() {
        return this.meta.app;
    }
    /**
     * Returns the list of application bootstrappers.
     */
    bootstrappers() {
        return [
            bootstrappers_1.HandleExceptions,
            bootstrappers_1.LoadEnvironmentVariables,
            bootstrappers_1.LoadConfiguration,
            bootstrappers_1.RegisterServiceProviders,
            bootstrappers_1.BootServiceProviders
        ];
    }
    /**
     * Handle an incoming console command for the given `input`.
     *
     * @param {Array} input - command line arguments (process.argv)
     *
     * @returns {Promise}
     */
    async run(input) {
        await this.bootstrap();
        await this.craft().run(input);
    }
    /**
     * Bootstrap the console application for Craft commands.
     */
    async bootstrap() {
        await this.app().bootstrapWith(this.bootstrappers());
        await this.commands();
    }
    /**
     * Returns the list of command class names to exclude when loading all
     * commands into the console application. This is useful for base
     * commands that provide shared functionality to other commands.
     *
     * @returns {Array}
     */
    excludedCommands() {
        return [];
    }
    /**
     * Register the console commands to the application.
     */
    async commands() {
        //
    }
    /**
     * Register all of the commands in the given `directories`.
     *
     * @param {Array} paths
     */
    async loadFrom(...paths) {
        await (0, collections_1.default)(paths)
            .unique()
            .filter(async (path) => {
            return await fs_1.default.exists(path);
        })
            .flatMap(async (path) => {
            return await fs_1.default.allFiles(path);
        })
            .map((commandFile) => {
            return this.resolve(commandFile); // TODO fix this typing issue in @supercharge/collections
        })
            .filter((command) => {
            return command instanceof console_1.Command && this.isNotExcluded(command);
        })
            .forEach((command) => {
            this.craft().add(command);
        });
    }
    /**
     * Requires the given file from disk.
     *
     * @param {String} commandFile
     *
     * @returns {Command}
     */
    async resolve(commandFile) {
        const CommandCtor = (0, goodies_1.esmRequire)(commandFile);
        return new CommandCtor();
    }
    /**
     * Determine whether the given `command` should not be excluded
     * from loading it to the list of available commands.
     *
     * @param {Command} command
     *
     * @returns {Boolean}
     */
    isNotExcluded(command) {
        return !this.shouldExclude(command);
    }
    /**
     * Determine whether the given `command` should be excluded
     * from loading it to the list of available commands.
     *
     * @param {Command} command
     *
     * @returns {Boolean}
     */
    shouldExclude(command) {
        return this.excludedCommands().includes(command.constructor.name);
    }
    /**
     * Returns a Craft console application instance.
     *
     * @returns {Craft}
     */
    craft() {
        if (!this.meta.craft) {
            this.meta.craft = new console_1.Application(this.app());
        }
        return this.meta.craft;
    }
}
exports.ConsoleKernel = ConsoleKernel;
