'use strict';
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ErrorHandler = void 0;
const http_error_1 = require("./http-error");
const goodies_1 = require("@supercharge/goodies");
const collections_1 = __importDefault(require("@supercharge/collections"));
class ErrorHandler {
    /**
     * Create a new error handler instance.
     */
    constructor(app) {
        this.app = app;
        this.reportCallbacks = [];
        this.register();
    }
    /**
     * The application’s logger instance.
     */
    logger() {
        return this.app.logger();
    }
    /**
     * The application’s view renderer.
     */
    view() {
        return this.app.make('view');
    }
    /**
     * Register the error handling callbacks. For example, to report error
     * upstream to an error tracking service, like Sentry or Bugsnag.
     */
    register() {
        //
    }
    /**
     * Register a reportable callback.
     *
     * @param  {Function} reportUsing
     *
     * @returns {ErrorHandler}
     */
    reportable(reportUsing) {
        return (0, goodies_1.tap)(this, () => {
            this.reportCallbacks.push(reportUsing);
        });
    }
    /**
     * Returns the default logging context.
     *
     * @param {HttpContext} ctx
     *
     * @returns {*}
     */
    context(_ctx) {
        return {};
    }
    /**
     * Handle the given error.
     *
     * @param {HttpContext} ctx
     * @param {Error} error
     */
    async handle(ctx, error) {
        const httpError = http_error_1.HttpError.wrap(error);
        await this.report(ctx, httpError);
        await this.render(ctx, httpError);
    }
    /**
     * Report an error.
     *
     * @param {HttpContext} ctx
     * @param {HttpError} error
     */
    async report(ctx, error) {
        await (0, collections_1.default)(this.reportCallbacks).forEach(async (reportCallback) => {
            await reportCallback(ctx, error);
        });
        this.logger().error(error.message, { ...this.context(ctx), error });
    }
    /**
     * Render the error into an HTTP response.
     *
     * @param {HttpContext} ctx
     * @param {HttpError} error
     */
    async render(ctx, error) {
        return ctx.request.wantsJson()
            ? this.renderJsonResponse(ctx, error)
            : await this.renderViewResponse(ctx, error);
    }
    /**
     * Creates a JSON response depending on the app’s environment.
     *
     * @param {HttpContext} ctx
     * @param {HttpError} error
     */
    renderJsonResponse(ctx, error) {
        const { message, stack, status: statusCode } = error;
        this.app.env().isProduction()
            ? ctx.response.status(statusCode).payload({ message })
            : ctx.response.status(statusCode).payload({ message, stack, statusCode });
    }
    /**
     * Creates an HTML response depending on the app’s environment.
     *
     * @param {HttpContext} ctx
     * @param {HttpError} error
     */
    async renderViewResponse(ctx, error) {
        if (await this.isMissingTemplateFor(error)) {
            this.logger().debug(`No error view template found at "${this.viewTemplateFor(error)}". Falling back to JSON response.`);
            return this.renderJsonResponse(ctx, error);
        }
        await ctx.response
            .status(error.status)
            .view(this.viewTemplateFor(error), { error });
    }
    /**
     * Determine whether a view template file is missing for the given `error`.
     *
     * @param {HttpError} error
     *
     * @returns {Boolean}
     */
    async isMissingTemplateFor(error) {
        return !await this.templateExistsFor(error);
    }
    /**
     * Determine whether a view template file exists for the given `error`.
     *
     * @param {HttpError} error
     *
     * @returns {Boolean}
     */
    async templateExistsFor(error) {
        return await this.view().exists(this.viewTemplateFor(error));
    }
    /**
     * Returns the view template file for the given `error`.
     *
     * @param {HttpError} error
     *
     * @returns {String}
     */
    viewTemplateFor(error) {
        return `errors/${error.status}`;
    }
}
exports.ErrorHandler = ErrorHandler;
