'use strict';
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HttpKernel = void 0;
const fs_1 = __importDefault(require("@supercharge/fs"));
const goodies_1 = require("@supercharge/goodies");
const http_1 = require("@supercharge/http");
const collections_1 = __importDefault(require("@supercharge/collections"));
const bootstrappers_1 = require("../bootstrappers");
class HttpKernel {
    /**
     * Create a new HTTP kernel instance.
     *
     * @param {Application} app
     */
    constructor(app) {
        this.server = new http_1.Server(this);
        this.meta = { app, bootedCallbacks: [] };
        this.register();
    }
    /**
     * Creates a new HTTP kernel instance for the given `app`.
     *
     * @param {Application} app
     */
    static for(app) {
        return new this(app);
    }
    /**
     * Register the booting or booted callbacks. For example, you may use a `booted`
     * callback to send a "ready" signal to PM2 (or any other process manager)
     * informing that your HTTP server is ready to accept incoming requests.
     */
    register() {
        //
    }
    /**
     * Returns the application’s global middleware stack. Every middleware
     * listed here runs on every request to the application.
     *
     * @returns {MiddlewareCtor[]}
     */
    middleware() {
        return [];
    }
    /**
     * Returns the application’s global middleware stack. Every middleware
     * listed here runs on every request to the application.
     *
     * @returns {MiddlewareCtor[]}
     */
    routeMiddleware() {
        return {};
    }
    /**
     * Returns the path to the HTTP controllers.
     *
     * @returns {String}
     */
    controllersLocation() {
        return this.app().resolveFromBasePath('app/http/controllers');
    }
    /**
     * Register a booted callback that runs after the HTTP server started.
     *
     * @returns {this}
     */
    booted(callback) {
        return (0, goodies_1.tap)(this, () => {
            this.bootedCallbacks().push(callback);
        });
    }
    /**
     * Returns the booted callbacks.
     *
     * @returns {Callback[]}
     */
    bootedCallbacks() {
        return this.meta.bootedCallbacks;
    }
    /**
     * Returns the app instance.
     */
    app() {
        return this.meta.app;
    }
    /**
     * Start the HTTP instance listening for incoming requests.
     *
     * @returns {Promise}
     */
    async startServer() {
        await this.bootstrap();
        await this.listen();
        await this.runBootedCallbacks();
    }
    /**
     * Bootstrap the application by loading environment variables, load the
     * configuration, register service providers into the IoC container
     * and ultimately boot the registered providers.
     */
    async bootstrap() {
        await this.app().bootstrapWith(this.bootstrappers());
        await this.server.bootstrap();
    }
    /**
     * Returns the list of application bootstrappers.
     */
    bootstrappers() {
        return [
            bootstrappers_1.HandleExceptions,
            bootstrappers_1.LoadEnvironmentVariables,
            bootstrappers_1.LoadConfiguration,
            bootstrappers_1.RegisterServiceProviders,
            bootstrappers_1.BootServiceProviders
        ];
    }
    /**
     * Returns an array of file paths to all controllers.
     *
     * @returns {String[]}
     */
    async controllerPaths() {
        return await fs_1.default.allFiles(this.controllersLocation());
    }
    /**
     * Start the HTTP server to listen on a local port.
     */
    async listen() {
        await this.server.start();
    }
    /**
     * Run the configured `booted` callbacks.
     */
    async runBootedCallbacks() {
        await this.runCallbacks(this.bootedCallbacks());
    }
    /**
     * Call the given kernal `callbacks`.
     *
     * @param {Callback[]} callbacks
     */
    async runCallbacks(callbacks) {
        await (0, collections_1.default)(callbacks).forEach(async (callback) => {
            await callback();
        });
    }
}
exports.HttpKernel = HttpKernel;
